#!/usr/bin/env python

import re


def encode_xml_entities(unescaped):
    """Makes a string safe for XML by substituting entities."""

    escaped = unescaped.replace('&', '&amp;')
    escaped = escaped.replace("'", '&apos;')
    escaped = escaped.replace('"', '&quot;')
    escaped = escaped.replace('<', '&lt;')
    escaped = escaped.replace('>', '&gt;')

    return escaped


def decode_xml_entities(escaped):
    """Converts XML entities back to ordinary characters."""

    unescaped = escaped.replace('&amp;', '&')
    unescaped = unescaped.replace('&apos;', "'")
    unescaped = unescaped.replace('&quot;', '"')
    unescaped = unescaped.replace('&lt;', '<')
    unescaped = unescaped.replace('&gt;', '>')

    return unescaped


def decode_hex_entity(escaped):
    """Unescapes hexadecimal character entities"""

    unescaped = escaped.group(0)[3:-1]
    unescaped = unichr(int(unescaped, 16))

    return unescaped


def decode_entities(escaped):
    """Unescapes hexadecimal and XML entities"""

    unescaped = re.sub('&#x\w{1,4};', decode_hex_entity, escaped)

    unescaped = decode_xml_entities(unescaped)

    return unescaped

